#! /usr/bin/python2

# This hal_manualtoolchange.py has been improved (from the version included
# with linuxcnc 2.8.2) to add tool names to the dialog box.
#
# The tool names are the remarks in the tooltable file
#
# Ideally we'd want to reload the tooltable file when linuxcnc reloads it
# (i.e. make sure that everything stays in sync).  But there is no way to
# reliably work out when that has happened.  So there is a new parameter
# to instruct hal_manualtoolchange when to reload the tooltable.
#
#
# Because there is no way to know for sure when linuxcnc has reloaded the
# tooltable; there is a new "reload_mode" hal parameter, used as follows:
#
# setp hal_manualtoolchange.reload_mode 0    # detect tooltable updates
# setp hal_manualtoolchange.reload_mode 1    # load the tooltable once
# setp hal_manualtoolchange.reload_mode 2    # load the tooltable every time M6 is used
#
# the way that mode 0 works is by checking if any of the numeric
# parameters in the tooltable has changed, and if they have to reload
# the tooltable.  (adding or removing tools from the table are
# detected as well).  The only thing that this will miss is an edit to
# just the remarks/toolnames.

import sys, os
import gettext
import copy

BASE = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), ".."))
gettext.install("linuxcnc", localedir=os.path.join(BASE, "share", "locale"), unicode=True)

import linuxcnc, hal

_after = None
def hal_in_background():
    global _after
    _after = None
    if not h.change:
        app.tk.call("set", "::tkPriv(button)", -1)
        return

    if (h.change_button):
        h.changed = True
        app.update()
        app.tk.call("set", "::tkPriv(button)", -1)
        stop_polling_hal_in_background()
        return

    _after = app.after(100, hal_in_background)

def poll_hal_in_background():
    global _after
    _after = app.after(100, hal_in_background)

def stop_polling_hal_in_background():
    global _after
    if _after: app.after_cancel(_after)
    _after = None

def do_change(n):
    if n:
        load_tooltable_if_needed(firsttime = False)
        tool_remark = get_tool_remark(n)
        message = _("Insert tool %d (%s) and click continue when ready") % (n, tool_remark)
    else:
        message = _("Remove the tool and click continue when ready")
    app.wm_withdraw()
    app.update()
    poll_hal_in_background()
    try:
        r = app.tk.call("nf_dialog", ".tool_change",
            _("Tool change"), message, "info", 0, _("Continue"))
    finally:
        stop_polling_hal_in_background()
    if isinstance(r, str): r = int(r)
    if r == 0:
        h.changed = True
    app.update()

tooltable_lines = None
tooltable_linuxcnc = None
def load_tooltable_if_needed(firsttime):
    global tooltable_lines
    global tooltable_linuxcnc

    try:
        if h.reload_mode == 1:
            # load only the first time
            load = firsttime
        elif h.reload_mode == 2:
            # reload every time
            load = True
        else: # default
            # reload if tooltable_linuxcnc is out of date
            s = linuxcnc.stat()
            s.poll()
            # linuxcnc uses the first list item to refer to the loaded tool - so we need to remove element 0
            tooltable_linuxcnc_current = copy.deepcopy(s.tool_table[1:])
            # and see if it has changed since last time
            load = firsttime
            if tooltable_linuxcnc != tooltable_linuxcnc_current:
                tooltable_linuxcnc = tooltable_linuxcnc_current
                load = True

        # finally, load tool table (if needed)
        if load:
            print("hal_manualtoolchange: load_tooltable(): load table")
            with open(path_tooltable, "r") as f:
                tooltable_lines = f.readlines()

    except Exception as e:
        # if anything fails...
        #     print the exception to the log
        #     and then ignore it to allow dialog box to appear
        print("hal_manualtoolchange: load_tooltable_if_needed() caused this exception")
        print(e)

def get_tool_remark(n):
    try:
        # check every line in the tooltable to find one that refers to tool n...
        for tool_entry in tooltable_lines:
            if tool_entry.startswith(_("T%s ") % n):
                # ... and return the remark
                tool_remark = tool_entry.split(";", 1)[1].strip()
                return tool_remark

    except Exception as e:
        # if anything fails...
        #     ignore it to allow dialog box to appear
        pass

    return ""

def find_tooltable_path(path_inifile):
    # work out the full path of the tooltable
    inifile = linuxcnc.ini(path_inifile)
    path_tooltable = inifile.find("EMCIO", "TOOL_TABLE").lstrip()
    if path_tooltable.startswith("/"):
        return path_tooltable
    elif path_tooltable.startswith("~"):
        return path_tooltable
    else:
        return os.path.join(os.path.dirname(path_inifile), path_tooltable)

try:
    path_inifile = os.environ.get("INI_FILE_NAME", "/dev/null")
    path_tooltable = find_tooltable_path(path_inifile)
except:
    pass


h = hal.component("hal_manualtoolchange")
h.newpin("number", hal.HAL_S32, hal.HAL_IN)
h.newpin("change", hal.HAL_BIT, hal.HAL_IN)
h.newpin("change_button", hal.HAL_BIT, hal.HAL_IN)
h.newpin("changed", hal.HAL_BIT, hal.HAL_OUT)
h.newparam("reload_mode", hal.HAL_S32, hal.HAL_RW)
h.ready()

import Tkinter, nf, rs274.options

app = Tkinter.Tk(className="AxisToolChanger")
app.wm_geometry("-60-60")
app.wm_title(_("AXIS Manual Toolchanger"))
rs274.options.install(app)
nf.start(app); nf.makecommand(app, "_", _)
app.wm_protocol("WM_DELETE_WINDOW", app.wm_withdraw)
lab = Tkinter.Message(app, aspect=500, text = _("\
This window is part of the AXIS manual toolchanger.  It is safe to close \
or iconify this window, or it will close automatically after a few seconds."))
lab.pack()

def withdraw():
    app.wm_withdraw()
    app.bind("<Expose>", lambda event: app.wm_withdraw())

app.after(10 * 1000, withdraw)

# load the tooltable now
load_tooltable_if_needed(firsttime = True)

try:
    while 1:
        change = h.change
        if change and not h.changed:
            do_change(h.number)
        elif not change:
            h.changed = False
        app.after(100)
        app.update()
except KeyboardInterrupt:
    pass
